// The MIT License
//
// Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Copyright (C) 2020 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Permission is hereby granted, free of charge, to any person obtaining a copy of
// this software and associated documentation files (the "Software"), to deal in
// the Software without restriction, including without limitation the rights to
// use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
// of the Software, and to permit persons to whom the Software is furnished to do
// so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all
// copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
// SOFTWARE.

import Foundation
import CMSData

final class KnowledgeEnvironmentStoreImpl : ObservableObject, KnowledgeEnvironmentStore {
    @Published var checklists: [Checklist] = []
    @Published var isLoading: Bool = false
    @Published var error: Error?

    @DI private var checklistService: ChecklistService

    static let shared = KnowledgeEnvironmentStoreImpl()
    static let preview = KnowledgeEnvironmentStoreImpl(checklists: Checklist.previewList())

    private var cancelBag = CancelBag()

    private init(checklists: [Checklist] = []) {
        self.checklists = checklists
    }

    func clear() {
        self.checklists = []
        self.error = nil
        self.isLoading = false
    }

    func fetchChecklists() {
        guard !isLoading && (checklists.isEmpty || error != nil) else {
            return
        }
        isLoading = true
        checklistService.fetchChecklists()
            .receiveOnMain()
            .sink(
                receiveCompletion: { [weak self] completion in
                    switch completion {
                    case .failure(let error):
                        self?.error = error
                    case .finished:
                        self?.error = nil
                    }
                    self?.isLoading = false
                },
                receiveValue: { [weak self] (checklists) in
                    self?.checklists = checklists
                })
            .store(in: &cancelBag)

    }

    func updateChecklist(_ checklist: Checklist, checklistItem: Checklist.Item) {
        guard let listIndex = checklists.firstIndex(where: { $0.id == checklist.id }),
              let itemIndex = checklists[listIndex].items.firstIndex(where: { $0.id == checklistItem.id }) else { return }

        let list = checklists[listIndex]
        var updatedList = checklists[listIndex].items
        updatedList[itemIndex] = checklistItem

        checklists[listIndex] = Checklist(id: list.id, title: list.title, description: list.description, items: updatedList)
    }
}
