// The MIT License
//
// Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Copyright (C) 2020 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Permission is hereby granted, free of charge, to any person obtaining a copy of
// this software and associated documentation files (the "Software"), to deal in
// the Software without restriction, including without limitation the rights to
// use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
// of the Software, and to permit persons to whom the Software is furnished to do
// so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all
// copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
// SOFTWARE.

import Foundation
import Combine
import SwiftUI
import MaternityServices
import ConfigurationServices
import GMAppData
import UserSettingsServices
import ErrorData

/// There are a lot of different views, who needs knowledge of the maternity data.
/// Therefor this is intended to be used as an EnvironmentObject.
/// It is a singleton, such that view models can initiate update requests without having access to the instance as an EnvironmentObject.
/// As long as we are using this class and try to avoid the direct use of the services in other places, then all views will receive the data updates.
final class MaternityDataEnvironmentStoreImpl : ObservableObject, MaternityDataEnvironmentStore {
    @DI private var userSettings: UserSettingsService
    @DI private var logger: Logger
    @DI private var dataCardService: DataCardService
    @DI private var configurationService: ConfigurationService

    @Published private (set) var maternities: [DataCard]?
    @Published private (set) var selectedMaternity: DataCard?
    @Published private (set) var dateOfBirthForSelectedMaternity: Date?

    private var loadingUpdatePublisher: AnyPublisher<Void, ServiceError>?

    static let shared = MaternityDataEnvironmentStoreImpl()

    private init() { }

    func changeSelectedMaternity(_ maternity: DataCard) -> AnyPublisher<Void, ServiceError> {
        if selectedMaternity?.id != maternity.id {
            logger.debug("Changing selected maternity to: \(maternity.id)")
        }
        userSettings.save(maternity.id, for: .selectedEpisodeOfCareId)
        selectedMaternity = maternity
        return loadDateOfBirthForSelectedMaternity()
    }

    func changeDateOfBirthForSelectedMaternity(_ date: Date?) -> AnyPublisher<Void, ServiceError> {
        guard let maternityId = self.selectedMaternity?.id else {
            return Fail(error: ServiceError.unexpectedError(nil))
                .eraseToAnyPublisher()
        }

        return configurationService
            .updateDateOfBirth(maternityId: maternityId, date: date)
            .receiveOnMain()
            .handleEvents(receiveOutput: { [weak self] (result: DateOfBirth) in
                self?.dateOfBirthForSelectedMaternity = result.dateOfBirth
            })
            .mapToVoid()
            .eraseToAnyPublisher()
    }

    /// Fetches all available data cards for user, sets the selected maternity and loads date of birth for the selected maternity.
    ///
    /// The implementation is protected against multiple calls at the same time, by returning the same publisher to all requesters while loading.
    func requestUpdate() -> AnyPublisher<Void, ServiceError> {
        if let workingPublisher = loadingUpdatePublisher {
            return workingPublisher
        } else {
            let workingPublisher = dataCardService.fetchDataCards()
                .receiveOnMain()
                .handleEvents(receiveOutput: { (cards: [DataCard]) in
                    self.maternities = cards
                })
                .flatMap({ cards -> AnyPublisher<Void, ServiceError> in
                    if let id = self.userSettings.get(key: .selectedEpisodeOfCareId),
                       let selectedMaternity = cards.first(where: { $0.episodeOfCareIdentifier == id }), self.selectedMaternity == nil {
                        return self.changeSelectedMaternity(selectedMaternity)
                    } else if let mostRecentMaternity = cards.mostRecentDataCard(), self.selectedMaternity == nil {
                        return self.changeSelectedMaternity(mostRecentMaternity)
                    } else {
                        // No changes to currently selected maternity. We're all good.
                        return Just(Void())
                            .setFailureType(to: ServiceError.self)
                            .eraseToAnyPublisher()
                    }
                })
                .handleEvents(
                    receiveCompletion: { [weak self] _ in
                        self?.loadingUpdatePublisher = nil
                    }
                )
                .mapToVoid()
                .eraseToAnyPublisher()

            loadingUpdatePublisher = workingPublisher
            return workingPublisher
        }
    }

    func clear() {
        selectedMaternity = nil
        dateOfBirthForSelectedMaternity = nil
        maternities = nil
    }

    private func loadDateOfBirthForSelectedMaternity() -> AnyPublisher<Void, ServiceError> {
        guard let maternityId = self.selectedMaternity?.episodeOfCareIdentifier else {
            logger.error("The app tried to request date of birth without having a selected maternity. This should **not** happen!")
            return Fail(error: ServiceError.unexpectedError(nil))
                .eraseToAnyPublisher()
        }
        return self.configurationService.dateOfBirth(maternityId: maternityId)
            .receiveOnMain()
            .handleEvents(receiveOutput: { [weak self] (dateOfBirth) in
                self?.dateOfBirthForSelectedMaternity = dateOfBirth.dateOfBirth
            })
            .mapToVoid()
            .eraseToAnyPublisher()
    }
}
