// The MIT License
//
// Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Copyright (C) 2020 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Permission is hereby granted, free of charge, to any person obtaining a copy of
// this software and associated documentation files (the "Software"), to deal in
// the Software without restriction, including without limitation the rights to
// use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
// of the Software, and to permit persons to whom the Software is furnished to do
// so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all
// copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
// SOFTWARE.

import Foundation
import TIM
import UserSettingsServices
import MaternityServices
import ConfigurationServices
import GMAppData
import ErrorData
import Combine

final class UserContext : UserContextProtocol {
    @DI private var timAuth: TIMAuth
    @DI private var timStorage: TIMDataStorage
    @DI private var userSettings: UserSettingsService
    @DI private var logger: Logger
    @DI private var maternityEnvironmentStore: MaternityDataEnvironmentStore
    @DI private var knowledgeEnvironmentStore: KnowledgeEnvironmentStore
    @DI private var configurationService: ConfigurationService

    var hasHandledOnboardingFlow: Bool {
        userSettings.get(key: .hasHandledOnboarding) ?? false
    }

    var isAuthenticated: Bool {
        timAuth.isLoggedIn
    }

    var userId: String? {
        timAuth.refreshToken?.userId
    }

    var hasUserRegistered: Bool {
        !timStorage.availableUserIds.isEmpty
    }

    func logout() {
        guard let userId = userId else {
            resetForUndefinedUserState()
            return
        }

        timAuth.logout()
        clearEnvironmentStoreStates()
        NotificationCenter.default.post(
            notification: Notification.Name.SceneDelegate.changeRootViewController,
            value: SceneDelegate.NotificationMessage.logout(userId: userId)
        )
    }

    func resetAllUserData() -> AnyPublisher<Void, ServiceError> {
        configurationService
            .deleteAllData()
            .receiveOnMain()
            .handleEvents(receiveCompletion: { (completion) in
                switch completion {
                case .finished:
                    if let userId = self.userId {
                        self.timAuth.logout()
                        self.timStorage.clear(userId: userId)
                    }
                    self.clearStoredUserSettings()
                    self.resetToOnboarding()
                default: break
                }
            })
            .eraseToAnyPublisher()
    }

    func setHasHandledOnboardingFlow(_ handled: Bool) {
        userSettings.save(handled, for: .hasHandledOnboarding)
    }

    private func clearEnvironmentStoreStates() {
        let userDataStores: [UserDataStore] = [
            self.maternityEnvironmentStore,
            self.knowledgeEnvironmentStore
        ]
        userDataStores.forEach({ $0.clear() })
    }

    private func resetForUndefinedUserState() {
        logger.critical("Tried to logout, but did not have a userId. Resetting all registered users and resetting to onboarding!")
        userSettings.remove(key: .selectedEpisodeOfCareId)
        timStorage.clearAllUsers()
        setHasHandledOnboardingFlow(false)
        clearEnvironmentStoreStates()
        clearStoredUserSettings()
        resetToOnboarding()
    }

    private func clearStoredUserSettings() {
        userSettings.remove(key: .selectedEpisodeOfCareId)

        if self.timStorage.availableUserIds.isEmpty {
            userSettings.remove(key: .hasHandledOnboarding)
        }
    }

    private func resetToOnboarding() {
        NotificationCenter.default.post(
            notification: Notification.Name.SceneDelegate.changeRootViewController,
            value: SceneDelegate.NotificationMessage.resetToOnboarding
        )
    }
}
