// The MIT License
//
// Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Copyright (C) 2020 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Permission is hereby granted, free of charge, to any person obtaining a copy of
// this software and associated documentation files (the "Software"), to deal in
// the Software without restriction, including without limitation the rights to
// use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
// of the Software, and to permit persons to whom the Software is furnished to do
// so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all
// copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
// SOFTWARE.

import Foundation
import Combine
import ErrorData
import GMAppData
import UIData
import MaternityServices
import CMSServices

final class HealthcareContactsServiceImpl: HealthcareContactsService {
    @DI private var maternityEnvironmentStore: MaternityDataEnvironmentStore
    @DI private var dictionaryService: DictionaryService
    
    func fetchGeneralPractitioner() -> AnyPublisher<[KeyValueModel], ServiceError> {
        let generalPractitioner = maternityEnvironmentStore.selectedMaternity?.personalDataSection?.generalPractitioner
        
        return dictionaryService.fetchDictionaryWithEmptyFallback()
            .map { dictionary -> [KeyValueModel] in
                [
                    KeyValueModel.dictionaryModel(
                        title: Localizable.journal_section_health_care_gp_1_name(),
                        primaryValue: generalPractitioner?.name,
                        dictionary: dictionary
                    ),

                    KeyValueModel.dictionaryModel(
                        title: Localizable.journal_section_health_care_gp_phone_number(),
                        primaryValue: generalPractitioner?.telephoneNumber,
                        dictionary: dictionary
                    ),
                    
                    KeyValueModel.dictionaryModel(
                        title: Localizable.journal_section_health_care_gp_address(),
                        primaryValue: generalPractitioner?.address?.streetAndNumber
                    ),
                    
                    KeyValueModel.dictionaryModel(
                        title: Localizable.journal_section_health_care_gp_postal_code(),
                        primaryValue: generalPractitioner?.address?.zip
                    ),
                    
                    KeyValueModel.dictionaryModel(
                        title: Localizable.journal_section_health_care_gp_city(),
                        primaryValue: generalPractitioner?.address?.city
                    )
                ]
            }
            .mapToServiceError()
            .eraseToAnyPublisher()
    }
    
    func fetchMidWife() -> AnyPublisher<[KeyValueModel], ServiceError> {
        let midwifeClinic = maternityEnvironmentStore.selectedMaternity?.midwifeClinicSection
        
        return dictionaryService.fetchDictionaryWithEmptyFallback()
            .map { dictionary -> [KeyValueModel] in
                [
                    KeyValueModel.dictionaryModel(
                        title: Localizable.journal_section_health_care_midwife_assigned_clinic(),
                        primaryValue: midwifeClinic?.assignedMidwifeClinic?.name,
                        dictionary: dictionary
                    ),

                    KeyValueModel.dictionaryModel(
                        title: Localizable.journal_section_health_care_midwife_craft_class_desired(),
                        primaryValue: (midwifeClinic?.parentalCraftClassDesired != nil ? YesNoEnum._true : YesNoEnum._false).description,
                        dictionary: dictionary
                    ),
                    
                    KeyValueModel.dictionaryModel(
                        title: Localizable.journal_section_health_care_midwife_consultation_form(),
                        primaryValue: midwifeClinic?.parentalCraftClassDesired?.consultationForm?.description,
                        dictionary: dictionary
                    ),
                    
                    KeyValueModel.dictionaryModel(
                        title: Localizable.journal_section_health_care_midwife_clinic_wishes(),
                        primaryValue: midwifeClinic?.midwifeConsultationWishes?.location,
                        dictionary: dictionary
                    ),
                    
                    KeyValueModel.dictionaryModel(
                        title: Localizable.journal_section_health_care_midwife_practitioner_wishes(),
                        primaryValue: midwifeClinic?.midwifeConsultationWishes?.midwife?.name.fullName,
                        dictionary: dictionary
                    )
                ]
            }
            .mapToServiceError()
            .eraseToAnyPublisher()
    }
    
    func fetchBirthplace() -> AnyPublisher<[KeyValueModel], ServiceError> {
        let birthPlaceSection = maternityEnvironmentStore.selectedMaternity?.birthPlaceSection
        
        return dictionaryService.fetchDictionaryWithEmptyFallback()
            .map { dictionary -> [KeyValueModel] in
                [
                    KeyValueModel.dictionaryModel(
                        title: Localizable.journal_section_health_care_birthplace_planned(),
                        primaryValue: birthPlaceSection?.preferredBirthplaceName,
                        dictionary: dictionary
                    ),
                    
                    KeyValueModel.dictionaryModel(
                        title: Localizable.journal_section_health_care_birthplace_desired(),
                        primaryValue: birthPlaceSection?.desiredBirthPlace,
                        dictionary: dictionary
                    )
                ]
            }
            .mapToServiceError()
            .eraseToAnyPublisher()
    }
    
    func fetchNurse() -> AnyPublisher<[KeyValueModel], ServiceError> {
        let healthNurse = maternityEnvironmentStore.selectedMaternity?.healthcareSection?.healthNurse

        return Just([
            KeyValueModel.dictionaryModel(
                title: Localizable.journal_section_health_care_nurse_ward(),
                primaryValue: maternityEnvironmentStore.selectedMaternity?.healthcareSection?.ward?.name
            ),
            
            KeyValueModel.dictionaryModel(
                title: Localizable.journal_section_health_care_nurse_nurse_name(),
                primaryValue: healthNurse?.name
            ),
            
            KeyValueModel.dictionaryModel(
                title: Localizable.journal_section_health_care_nurse_nurse_phonenumber(),
                primaryValue: healthNurse?.phone
            )
        ])
        .mapToServiceError()
        .eraseToAnyPublisher()
    }
}
