// The MIT License
//
// Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Copyright (C) 2020 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Permission is hereby granted, free of charge, to any person obtaining a copy of
// this software and associated documentation files (the "Software"), to deal in
// the Software without restriction, including without limitation the rights to
// use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
// of the Software, and to permit persons to whom the Software is furnished to do
// so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all
// copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
// SOFTWARE.

import SwiftUI
import LocalAuthentication
import ThePerfectApp

struct LoginView: View, ScreenTrackable {
    let userId: String

    @DI var logger: Logger
    @ObservedObject var viewModel: ViewModel
    @State var keyboardHeight: CGFloat = 0
    
    init(userId: String, presentedFromLogout: Bool) {
        self.viewModel = ViewModel(
            userId: userId,
            presentedFromLogout: presentedFromLogout
        )
        self.userId = userId
    }
    
    var body: some View {
        BubblesBackgroundView()
            .overlay(
                VStack {
                    PINTitleView(
                        pinCodeLength: viewModel.pinCodeLength,
                        isBiometricLoginAvailable: viewModel.isBiometricLoginAvailable,
                        title: Localizable.login_title(),
                        infoText: viewModel.loginText,
                        wrongCodeText: Localizable.login_error_wrong_pin_code(),
                        pinCode: $viewModel.pinCode,
                        isLoading: $viewModel.isLoading,
                        wrongCode: $viewModel.wrongCode,
                        biometricLoginWasActivated: $viewModel.biometricLoginWasActivated,
                        shouldResetWrongPinCodeState: $viewModel.shouldResetWrongPinCodeState
                    )
                    .padding()
                    Button(Localizable.login_forgot_password()) {
                        viewModel.forgotPassword = true
                    }
                    .font(FontStyle.info.scaledFont())
                    .foregroundColor(R.color.text.color())
                    .padding([.bottom])
                    .disabled(viewModel.isLoading)
                    if #available(iOS 14, *) {
                        // All good
                    } else {
                        Spacer(minLength: Metrics.Sizes.numberKeyboardHeight)
                    }
                }
                .padding()
            )
            .onAppear(perform: {
                let shouldAskForBioLogin: Bool = viewModel.isBiometricLoginAvailable &&
                    !viewModel.presentedFromLogout &&
                    !viewModel.biometricLoginWasActivated

                if shouldAskForBioLogin {
                    DispatchQueue.main.asyncAfter(deadline: .now() + .milliseconds(300)) {
                        logger.info("Requesting auto login with biometric.")
                        viewModel.biometricLoginWasActivated = true
                    }
                }
            })
            .screenTracking(screenTrackingName)
        viewModel.alertViewsForErrorStates()
        VStack { }
            .sheetOnBackground(isPresented: $viewModel.forgotPassword, content: {
                OnboardingConsentView()
            })
            .alertOnBackground(isPresented: $viewModel.failedToLoadInitialData, alert: {
                Alert(
                    title: Text(Localizable.login_loading_data_failed_alert_title()),
                    message: Text(Localizable.login_loading_data_failed_alert_description()),
                    dismissButton: .default(Text(Localizable.common_ok())) {
                        // Check for TPA updates if dialog was dismissed by this error.
                        if App.buildConfiguration.isTPADistribution {
                            logger.info("Checking for TPA update...")
                            TPA.shared().checkForUpdate()
                        }
                    }
                )
            })
    }
}

struct LoginView_Previews: PreviewProvider {
    static var previews: some View {
        LoginView(userId: "<userId>", presentedFromLogout: false)
    }
}
