// The MIT License
//
// Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Copyright (C) 2020 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Permission is hereby granted, free of charge, to any person obtaining a copy of
// this software and associated documentation files (the "Software"), to deal in
// the Software without restriction, including without limitation the rights to
// use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
// of the Software, and to permit persons to whom the Software is furnished to do
// so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all
// copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
// SOFTWARE.

import CMSData
import CMSServices
import ErrorData
import GMAppData
import MaternityServices
import MapKit
import SwiftUI

extension AppointmentDetailView {
    final class ViewModel: ObservableObject {
        
        @Published var isDirectionsActionSheetPresented: Bool = false
        @Published var dictionary = CMSDictionary.empty()
        @Published var activity: Activity?
        @Published var state: DataLoadingState = .initial
        @DI var maternityEnvironmentStore: MaternityDataEnvironmentStore

        var isPlannedOrFinished: Bool {
            activity?.status == .planned || activity?.status == .finished
        }

        var badgeText: String? {
            activity?.status == .finished ? Localizable.appointment_details_finished_badge() : Localizable.appointment_details_planned_badge()
        }

        var subtitle: String? {
            guard let date = activity?.plannedTime?.from, activity?.status == .finished else {
                return Localizable.appointment_week_label(activity?.gestationWeek ?? Localizable.common_missing_data())
            }
            return AppointmentsView.dateFormatter.string(from: date)
        }

        @DI private var dictionaryService: DictionaryService
        @DI private var carePlanService: CarePlanService
        @DI private var logger: Logger

        private var activityId: String
        private var cancelBag = CancelBag()

        init(activityId: String) {
            self.activityId = activityId
        }

        init(activity: Activity) {
            self.state = .data
            self.activityId = activity.id ?? "<missing_id>"
            self.activity = activity
        }

        func fetchDictionary() {
            dictionaryService
                .fetchDictionaryWithEmptyFallback()
                .sink { (dictionary: CMSDictionary) in
                    self.dictionary = dictionary
                }
                .store(in: &cancelBag)
        }

        func fetchActivity() {
            guard let maternityId = maternityEnvironmentStore.selectedMaternity?.episodeOfCareIdentifier, state.allowsDataFetch else {
                if state.allowsDataFetch {
                    self.state = .noSelectedMaternityEmpty()
                }
                return
            }
            self.state = .loading
            carePlanService
                .fetchCarePlan(maternityId: maternityId)
                .compactMap { [weak self] (carePlan) -> Activity? in
                    carePlan.activities?.first(where: { $0.id == self?.activityId })
                }
                .receiveOnMain()
                .sink(receiveCompletion: { [weak self] (completion) in
                    switch completion {
                    case .failure(let error):
                        self?.state = .error(error)
                    case .finished:
                        if self?.activity == nil && self != nil {
                            self?.logger.error("The activity should never be nil when this call has finished. Something went wrong when searching for the right activity ID.")
                            self?.state = .error(ServiceError.unexpectedError(nil))
                        }
                    }
                },
                receiveValue: { (activity) in
                    self.activity = activity
                    self.state = .data
                }).store(in: &cancelBag)

        }
    }
}
