// The MIT License
//
// Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Copyright (C) 2020 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Permission is hereby granted, free of charge, to any person obtaining a copy of
// this software and associated documentation files (the "Software"), to deal in
// the Software without restriction, including without limitation the rights to
// use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
// of the Software, and to permit persons to whom the Software is furnished to do
// so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all
// copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
// SOFTWARE.

import SwiftUI
import GMAppData

struct ActivitiesTimeline<FooterView: View>: View {
    let activities: [Activity]
    let dueDate: Date?
    let labelTitle: String
    let labelColor: Color
    let isFirstInTimeline: Bool
    let backgroundColor: Color
    let lineColor: Color
    let footerView: FooterView

    private let topSpace: CGFloat = 20

    var body: some View {
        CompatibleVStack(spacing: 0) {
            if isFirstInTimeline {
                Rectangle()
                    .fill(Color.clear)
                    .frame(height: topSpace) //Space
            } else {
                HStack {
                    VerticalLineView(color: lineColor)
                        .frame(height: topSpace)
                    Spacer()
                }
                .padding(.leading)
            }

            if !activities.isEmpty {
                ActivitiesTimelineLabel(
                    title: labelTitle,
                    color: labelColor
                )
                .padding(.leading, 8)
            }
            ForEach(activities.indices) { index in
                let topPadding: CGFloat = index == 0 ? 20 : 8
                let activity: Activity = activities[index]
                let nextActivity: Activity? = activities[safe: index + 1]
                if let dueDate = dueDate, dueDate.isBetween(date1: activity.sortDate, date2: nextActivity?.sortDate) {
                    ActivityTimelineEntryView(
                        lineColor: lineColor,
                        dotColor: R.color.text.color(),
                        topPadding: topPadding,
                        content: {
                        DueDateCardView(dueDate: dueDate)
                    })
                }
                ActivityTimelineEntryView(
                    lineColor: lineColor,
                    topPadding: topPadding,
                    content: {
                    ActivityEntryCardView(
                        activity: activity
                    )
                })
            }
            footerView
        }
        .background(backgroundColor)
    }

    static func isDueDateBetween(_ dueDate: Date, activity1: Activity?, activity2: Activity?) -> Bool {
        dueDate.isBetween(date1: activity1?.sortDate, date2: activity2?.sortDate)
    }
}

struct ActivitiesTimeline_Previews: PreviewProvider {
    static var previews: some View {
        ScrollView {
            VStack(spacing: 0) {
                ActivitiesTimeline(
                    activities: [loadPreviewCarePlan().activities!.first!],
                    dueDate: nil,
                    labelTitle: "Title 1",
                    labelColor: .blue,
                    isFirstInTimeline: true,
                    backgroundColor: .gray,
                    lineColor: .pink,
                    footerView: HStack {
                        VerticalLineView(color: .pink)
                            .frame(height: 30)
                        Spacer()
                    }.padding(.leading)
                )
                ActivitiesTimeline(
                    activities: loadPreviewCarePlan().activities!,
                    dueDate: nil,
                    labelTitle: "Title 2",
                    labelColor: .blue,
                    isFirstInTimeline: false,
                    backgroundColor: .white,
                    lineColor: .pink,
                    footerView: Rectangle().fill(Color.pink).frame(size: 50)
                )
            }
        }
    }
}
