// The MIT License
//
// Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Copyright (C) 2020 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Permission is hereby granted, free of charge, to any person obtaining a copy of
// this software and associated documentation files (the "Software"), to deal in
// the Software without restriction, including without limitation the rights to
// use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
// of the Software, and to permit persons to whom the Software is furnished to do
// so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all
// copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
// SOFTWARE.

import SwiftUI

struct AppointmentsView: View, ScreenTrackable {
    @ObservedObject var viewModel: ViewModel
    @EnvironmentObject var maternityEnvironmentStore: MaternityDataEnvironmentStoreImpl

    @State private var didFinishInitialScrolling = false
    @State private var scrollOffset: CGFloat = 0
    @State private var infoViewBannerHeight: CGFloat = 0

    private let scrollDelayMilliseconds: Int = 500
    private let scrollId: String = "plannedActivities"
    private let scrollCoordinateSpace: String = "scroll"
    private let bottomIllustrationSize: CGSize = R.image.illustrations.bottom_appointments()!.size

    init(viewModel: ViewModel = ViewModel()) {
        self.viewModel = viewModel
    }

    var body: some View {
        NavigationView {
            GeometryReader { geo in
                ScrollView(viewModel.state.scrollAxis) {
                    DataLoadingView(
                        state: viewModel.state,
                        contentHeightForNoDataState: geo.size.height,
                        retryAction: { viewModel.fetchData() },
                        content: {
                            viewContent(safeAreaTop: geo.safeAreaInsets.top)
                        })
                        .padding(.bottom, infoViewBannerHeight)
                }
                .coordinateSpace(name: scrollCoordinateSpace)
                .overlay(
                    VStack {
                        Spacer(minLength: 0)
                        InfoView(text: Localizable.appointment_info())
                            .onSizeChanged(InfoViewSizeReaderKey.self, perform: { value in
                                infoViewBannerHeight = value.height
                            })
                    }
                )
            }
            .screenTracking(screenTrackingName)
            .navigationBarTitle(Text(Localizable.bottom_navigation_title_appointments()), displayMode: .automatic)
            .onReceive(maternityEnvironmentStore.$selectedMaternity, perform: { _ in
                // Reset scroll parameters on maternity change
                scrollOffset = 0
                didFinishInitialScrolling = false

                // And reload!
                viewModel.fetchData(force: true)
            })
        }
        .navigationViewStyle(StackNavigationViewStyle())
    }

    @ViewBuilder
    private func viewContent(safeAreaTop: CGFloat) -> some View {
        if #available(iOS 14.0, *) {
            ScrollViewReader { reader in
                scrollViewContent
                    .onAppear {
                        self.scrollToFirstFutureActivityIfNeeded(reader: reader)
                    }
            }
        } else {
            scrollViewContent
                .introspectScrollView { scrollview in
                    guard !didFinishInitialScrolling && scrollOffset > 0 else { return }
                    // Set this immediately to avoid potential multiple scroll invocations
                    didFinishInitialScrolling = true

                    DispatchQueue.main.asyncAfter(deadline: .now() + .milliseconds(scrollDelayMilliseconds)) {
                        // This does not give us the ideal scrolling position, but it is close enough for iOS 13!
                        // (several hours went into this, and this is the closest I'll get without hardcoding random values into the code!)
                        scrollview.setContentOffset(CGPoint(x: 0, y: scrollOffset - safeAreaTop), animated: true)
                    }
                }
        }
    }

    @ViewBuilder
    private var scrollViewContent: some View {
        VStack(spacing: 0) { // Using default VStack to make sure that all sections are present (we are using lazy stacks inside the components)
            ActivitiesTimeline(
                activities: viewModel.statusSet.finished,
                dueDate: viewModel.dueDateIfInBetween(viewModel.statusSet.finished, openEnd: false),
                labelTitle: Localizable.past_appointments_title(),
                labelColor: R.color.activitiesFinishedLine.color(),
                isFirstInTimeline: true,
                backgroundColor: R.color.viewBackgroundSecondary.color(),
                lineColor: R.color.activitiesFinishedLine.color(),
                footerView: HStack {
                    VerticalLineView(color: R.color.activitiesFinishedLine.color())
                        .frame(height: 12)
                        .padding(.leading)
                    Spacer()
                }
            )
            scrollableElement
            ActivitiesTimeline(
                activities: viewModel.statusSet.planned,
                dueDate: viewModel.dueDateIfInBetween(viewModel.statusSet.planned, openEnd: true),
                labelTitle: Localizable.future_appointments_title(),
                labelColor: R.color.primary.color(),
                isFirstInTimeline: false,
                backgroundColor: R.color.viewBackground.color(),
                lineColor: R.color.activitiesUnfinishedLine.color(),
                footerView: scrollFooterView
            )
        }
    }

    @ViewBuilder
    private var scrollFooterView: some View {
        HStack {
            VerticalLineView(color: R.color.activitiesUnfinishedLine.color())
                .padding(.leading)
            Spacer()
        }
        .frame(width: UIScreen.main.bounds.width)
        .frame(height: bottomIllustrationSize.height)
        .background(
            BottomScrollIllustration(image: R.image.illustrations.bottom_appointments)
        )
    }

    @ViewBuilder
    private var scrollableElement: some View {
        if #available(iOS 14, *) {
            EmptyView()
                .id(scrollId)
        } else {
            GeometryReader { reader in
                Color.clear
                    .frame(size: 0)
                    .onAppear(perform: {
                        scrollOffset = reader.frame(in: .named(scrollCoordinateSpace)).origin.y
                    })
            }
            .frame(size: 0)
        }
    }

    @available(iOS 14.0, *)
    private func scrollToFirstFutureActivityIfNeeded(reader: ScrollViewProxy) {
        guard !didFinishInitialScrolling else { return }

        DispatchQueue.main.asyncAfter(deadline: .now() + .milliseconds(scrollDelayMilliseconds)) {
            withAnimation {
                reader.scrollTo(scrollId, anchor: .top)
            }
        }
        didFinishInitialScrolling = true
    }
}

private struct InfoViewSizeReaderKey: SizeReaderKey {
    static var defaultValue: CGSize = .zero
}

struct AppointmentsView_Previews: PreviewProvider {
    static var previews: some View {
        AppointmentsView(
            viewModel: AppointmentsView.PreviewViewModel()
        )
        .environmentObject(MaternityDataEnvironmentStoreImpl.shared)
    }
}
