// The MIT License
//
// Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Copyright (C) 2020 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Permission is hereby granted, free of charge, to any person obtaining a copy of
// this software and associated documentation files (the "Software"), to deal in
// the Software without restriction, including without limitation the rights to
// use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
// of the Software, and to permit persons to whom the Software is furnished to do
// so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all
// copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
// SOFTWARE.

import MaternityServices
import GMAppData
import SwiftUI

extension AppointmentsView {
    class ViewModel: ObservableObject {
        @DI private var userContext: UserContext
        @DI private var carePlanService: CarePlanService

        @Published var state = DataLoadingState.initial
        @Published var statusSet: ActivityStatusSet = .empty()

        @DI var maternityEnvironmentStore: MaternityDataEnvironmentStore

        private var cancelBag = CancelBag()

        func fetchData(force: Bool = false) {
            // This endpoint is really expensive, so we will only load data once!
            guard let maternityId = maternityEnvironmentStore.selectedMaternity?.episodeOfCareIdentifier, (state.allowsDataFetch || force) else {
                if state.allowsDataFetch {
                    state = .noSelectedMaternityEmpty()
                }
                return
            }

            state = .loading
            carePlanService
                .fetchCarePlan(maternityId: maternityId)
                .map { carePlan -> ActivityStatusSet in
                    carePlan.activities?.statusSet() ?? .empty()
                }
                .receiveOnMain()
                .sink { [weak self] completion in
                    switch completion {
                    case .failure(let error):
                        self?.state = .error(error)
                    case .finished:
                        if self?.statusSet.isEmpty == true {
                            self?.state = .empty(Localizable.careplan_activities_empty_state_title(), Localizable.careplan_activities_empty_state_description())
                        } else {
                            self?.state = .data
                        }
                    }
                } receiveValue: { [weak self] statusSet in
                    self?.statusSet = statusSet
                }
                .store(in: &cancelBag)
        }
        
        func dueDateIfInBetween(_ activities: [Activity], openEnd: Bool) -> Date? {
            let activityDates = activities.compactMap({ $0.sortDate })
            let first = activityDates.min()
            let last = openEnd ? nil : activityDates.max()

            let dueDate = maternityEnvironmentStore.selectedMaternity?.dueDate
            return dueDate?.isBetween(date1: first, date2: last) == true ? dueDate : nil
        }
    }

    final class PreviewViewModel: ViewModel {
        override func fetchData(force: Bool = false) {
            statusSet = loadPreviewCarePlan()
                .activities?.statusSet() ?? .empty()
            self.state = .data
        }
    }
}
