// The MIT License
//
// Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Copyright (C) 2020 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Permission is hereby granted, free of charge, to any person obtaining a copy of
// this software and associated documentation files (the "Software"), to deal in
// the Software without restriction, including without limitation the rights to
// use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
// of the Software, and to permit persons to whom the Software is furnished to do
// so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all
// copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
// SOFTWARE.

import Foundation
import MaternityServices
import UIData
import Combine
import ErrorData

extension JournalMeasurementsView {
    class ViewModel: ObservableObject, JournalTableContainerViewModel {
        @Published var chartValues: [MeasurementsLineChart.Value] = []
        // Customer decided to wait with the normal area for now. Code is kept for possible future use.
        @Published var chartArea = MeasurementsLineChart.Area(upper: [], lower: [], middle: [])
        @Published var chartXUnit = ""
        @Published var chartYUnit = ""
        @Published var state: DataLoadingState = .initial
        @Published var bodyMeasureValues: [KeyValueModel] = []

        @Published var titleColumn: DataTableColumn = DataTableColumn(width: 0, cells: [])
        @Published var valueColumns: [DataTableColumn] = []
        @Published var shouldNavigateToActivity: Bool = false
        var activityIdForNavigation: String?

        @DI private var observationTableService: ObservationTableService
        @DI private var bodyMeasureService: BodyMeasureService
        @DI private var maternityEnvironmentStore: MaternityDataEnvironmentStore
        
        private var cancelBag = CancelBag()

        func fetchData() {
            guard let maternityId = maternityEnvironmentStore.selectedMaternity?.episodeOfCareIdentifier else {
                state = .noSelectedMaternityEmpty()
                return
            }
            guard state.allowsDataFetch else {
                return
            }
            state = .loading
            bodyMeasureService.fetchBodyMeasureData()
                .zip(observationTableService.fetchClinicalTable(maternityId: maternityId, navigateToActivity: navigateToActivity))
                .receiveOnMain()
                .sink(
                    receiveCompletion: { [weak self] (completion) in
                        switch completion {
                        case .failure(let error):
                            self?.state = .error(error)
                        case .finished:
                            break
                        }
                    },
                    receiveValue: { [weak self] bodyMeasureData, tableResult in
                        self?.state = .data
                        self?.bodyMeasureValues = bodyMeasureData
                        self?.chartValues = tableResult.pubisMeasurements
                            .map({ MeasurementsLineChart.Value(
                                x: $0.maternityWeek,
                                y: $0.value
                            )})

                        self?.chartXUnit = Localizable.common_unit_weeks()
                        self?.chartYUnit = tableResult.pubisMeasurements.first?.unit ?? ""
                        self?.titleColumn = DataTableColumn(width: 140, cells: tableResult.tableColumnSet.titleColumn)
                        self?.valueColumns = tableResult.tableColumnSet.valueColumns.map({ DataTableColumn(width: 100, cells: $0) })
                    }
                ).store(in: &cancelBag)
        }
    }

    class PreviewViewModel: ViewModel {
        override func fetchData() {
            chartValues = MeasurementsLineChart.Value.mockList()
            chartArea = MeasurementsLineChart.Area.mock()
            chartXUnit = "uger"
            chartYUnit = "cm"
            state = .data
            bodyMeasureValues = [KeyValueModel(title: "Preview", primaryValue: "Test")]
            valueColumns = ScrollableDataTable.previewColumns()
            titleColumn = ScrollableDataTable.previewTitleColumn()
        }
    }
}
