// The MIT License
//
// Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Copyright (C) 2020 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Permission is hereby granted, free of charge, to any person obtaining a copy of
// this software and associated documentation files (the "Software"), to deal in
// the Software without restriction, including without limitation the rights to
// use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
// of the Software, and to permit persons to whom the Software is furnished to do
// so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all
// copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
// SOFTWARE.

import SwiftUI
import CMSData

struct ArticleDetailView: View, ScreenTrackable {
    let article : Article
    let highlightSectionId: Int?
    @State private var didFinishHighlightScroll: Bool = false
    @State private var didAppear: Bool = false

    private let dateFormatter: DateFormatter

    init(article: Article, highlightSectionId: Int? = nil) {
        self.article = article
        self.highlightSectionId = highlightSectionId
        self.dateFormatter = DateFormatter()
        self.dateFormatter.timeStyle = .none
        self.dateFormatter.dateStyle = .medium
    }

    var body: some View {
        ScrollView {
            if #available(iOS 14, *) {
                ScrollViewReader { reader in
                    content
                        .onAppear(perform: {
                            scrollToHighlightedSectionIfNeeded(reader: reader)
                        })
                }
            } else {
                content
            }
        }
        .background(R.color.viewBackground.color())
        .navigationBarTitle(Text(article.title), displayMode: .inline)
        .screenTracking(screenTrackingName)
        .onAppear(perform: {
            // This is needed avoid crash on scroll to highlighted 🤯
            self.didAppear = true
        })
    }

    private var content: some View {
        VStack(alignment: .leading, spacing: 16) {
            if let description = article.description {
                Text(description)
                    .applyTextStyle(fontStyle: .body)
                    .padding(.vertical)
            }

            ForEach(article.sections) { section in
                ArticleSectionCard(
                    articleSection: section,
                    autoExpand: highlightSectionId == section.id
                )
            }

            HStack {
                Spacer()
                Text(Localizable.article_last_updated_on(dateFormatter.string(from: article.lastUpdated)))
                    .applyTextStyle(fontStyle: .info, textColor: R.color.greyText.color())
                Spacer()
            }
        }
        .padding()
    }

    @available(iOS 14.0, *)
    private func scrollToHighlightedSectionIfNeeded(reader: ScrollViewProxy) {
        guard let highlightedId = highlightSectionId, !didFinishHighlightScroll && didAppear else { return }

        DispatchQueue.main.asyncAfter(deadline: .now() + .milliseconds(300)) {
            withAnimation {
                reader.scrollTo(highlightedId, anchor: .top)
            }
        }
        didFinishHighlightScroll = true
    }
}

struct ArticleDetailView_Previews: PreviewProvider {
    static var previews: some View {
        NavigationView {
            ArticleDetailView(
                article: CMSData.Category.mockList().first!.articles.first!
            )
        }
    }
}
