// The MIT License
//
// Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Copyright (C) 2020 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Permission is hereby granted, free of charge, to any person obtaining a copy of
// this software and associated documentation files (the "Software"), to deal in
// the Software without restriction, including without limitation the rights to
// use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
// of the Software, and to permit persons to whom the Software is furnished to do
// so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all
// copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
// SOFTWARE.

import UIKit
import SwiftUI

struct KnowledgeContentView: View, ScreenTrackable {
    @ObservedObject var viewModel: ViewModel = ViewModel()
    @EnvironmentObject var environmentStore: KnowledgeEnvironmentStoreImpl
    let preferredNavigationBarDisplayMode: NavigationBarItem.TitleDisplayMode
    private var searchBarProvider: SearchBarProvider?
    
    private let searchBarContainerHeight: CGFloat = Metrics.Sizes.textFieldHeight +
        Metrics.Spacing.searchBarTopBottomMargin * 2
    
    init(preferredNavigationBarDisplayMode: NavigationBarItem.TitleDisplayMode) {
        self.preferredNavigationBarDisplayMode = preferredNavigationBarDisplayMode
        self.searchBarProvider = SearchBarProvider(
            searchQuery: $viewModel.searchQuery,
            searchFieldHasFocus: $viewModel.searchFieldHasFocus
        )
    }
    
    var body: some View {
        GeometryReader { geo in
            DataLoadingView(
                state: viewModel.state,
                animateStateChanges: true,
                retryAction: {
                    if viewModel.isSearching {
                        self.viewModel.searchForCurrentQuery()
                    } else {
                        self.viewModel.fetchData()
                    }
                },
                content: {
                    ScrollView {
                        if viewModel.isSearching && !viewModel.searchQuery.isEmpty {
                            KnowledgeViewSearchResultsView(
                                searchQuery: viewModel.searchQuery,
                                searchCategories: viewModel.searchResults
                            )
                            .padding()
                        } else {
                            CompatibleVStack(spacing: 0) {
                                KnowledgeViewChecklistSection()
                                KnowledgeViewCategoriesSection(categories: viewModel.categories)
                                Text(Localizable.knowledge_created_by_footer())
                                    .applyTextStyle(fontStyle: .info, textColor: R.color.greyText.color())
                                    .padding(.bottom)
                                    .multilineTextAlignment(.center)
                                BottomScrollIllustration(
                                    image: R.image.illustrations.bottom_knowledge
                                )
                            }
                        }
                    }
                }
            )
            .frame(width: geo.size.width, height: geo.size.height) // Necessary to layout the navigation bar + search controller correctly on iOS 13 🤯
            .overlay(
                ViewControllerResolver { (parent) in
                    parent.navigationItem.searchController = searchBarProvider?.searchController
                }.opacity(0)
            )
            .navigationBarTitle(
                Text(Localizable.bottom_navigation_title_knowledge()),
                displayMode: preferredNavigationBarDisplayMode
            )
            .protectedAppearanceEvents(onAppear: viewModel.fetchData)
            .screenTracking(screenTrackingName)
        }
    }
}

struct KnowledgeContentView_Previews: PreviewProvider {
    static var previews: some View {
        KnowledgeContentView(preferredNavigationBarDisplayMode: .automatic)
            .environmentObject(KnowledgeEnvironmentStoreImpl.preview)
    }
}
