// The MIT License
//
// Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Copyright (C) 2020 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Permission is hereby granted, free of charge, to any person obtaining a copy of
// this software and associated documentation files (the "Software"), to deal in
// the Software without restriction, including without limitation the rights to
// use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
// of the Software, and to permit persons to whom the Software is furnished to do
// so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all
// copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
// SOFTWARE.

import SwiftUI

struct RateTheAppView: View, ScreenTrackable {
    @ObservedObject private var viewModel: ViewModel = ViewModel()
    @Environment(\.presentationMode) var presentationMode
    @Binding var isPresentedAsSheet: Bool

    var body: some View {
        ScrollView {
            VStack {
                CircleImageHeaderView(
                    title: viewModel.title,
                    text: Localizable.more_rating_description(),
                    imageUrl: nil,
                    image: Image(R.image.icons.more.rate.name)
                        .renderingMode(.original)
                        .resizable()
                )
                .padding()

                BottomContainerCardView {
                    VStack(spacing: 20) {
                        VStack(alignment: .leading, spacing: 8) {
                            Text(Localizable.rate_the_app_question_title())
                                .applyTextStyle(fontStyle: .h2)
                            Text(Localizable.rate_the_app_question_description())
                            .applyTextStyle(fontStyle: .body)
                            .fixedSize(horizontal: false, vertical: true)
                        }
                        SmileyRateView(rating: $viewModel.smileyRating)
                            .padding(.horizontal)
                            .frame(height: 50)
                        VStack(alignment: .leading) {
                            Text(Localizable.rate_the_app_comment())
                                .applyTextStyle(fontStyle: .h4)
                            SUITextView(
                                text: $viewModel.text,
                                placeholderText: Localizable.rate_the_app_eventual_comment_hint()
                            )
                            .frame(minHeight: 130)
                            .overlay(
                                RoundedRectangle(cornerRadius: 4)
                                    .stroke(viewModel.validationFailed ? R.color.pinWrong.color() : R.color.text.color(),
                                            lineWidth: 1
                                    )
                            )
                            if viewModel.validationFailed {
                                Text(Localizable.rate_the_app_comment_required())
                                    .applyTextStyle(fontStyle: .info, textColor: R.color.pinWrong.color())
                            }

                        }
                        LoadingButton(
                            title: Localizable.rate_the_app_send_button_text(),
                            isLoading: viewModel.isSending,
                            action: {
                                App.topViewController?.view.endEditing(true)
                                viewModel.sendFeedback()
                            }
                        )
                        .padding(.top)
                    }
                    .padding()
                }
            }
            .alertOnBackground(isPresented: $viewModel.failedToSend, alert: {
                Alert(
                    title: Text(Localizable.common_alert_title_error()),
                    message: Text(Localizable.rate_the_app_failed()),
                    dismissButton: .default(Text(Localizable.common_ok()))
                )
            })
            .alertOnBackground(isPresented: $viewModel.didSend, alert: {
                Alert(
                    title: Text(Localizable.rate_the_app_success()),
                    message: Text(Localizable.rate_the_app_success_message()),
                    dismissButton: .default(Text(Localizable.common_cta_close()), action: {
                        presentationMode.wrappedValue.dismiss()
                        isPresentedAsSheet = false
                    })
                )
            })
        }
        .screenTracking(screenTrackingName)
        .onDisappear(perform: viewModel.reset)
        .navigationBarTitle(Text(viewModel.title), displayMode: .inline)
    }
}

struct RateTheAppView_Previews: PreviewProvider {
    static var previews: some View {
        RateTheAppView(isPresentedAsSheet: .constant(false))

    }
}
