// The MIT License
//
// Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Copyright (C) 2020 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Permission is hereby granted, free of charge, to any person obtaining a copy of
// this software and associated documentation files (the "Software"), to deal in
// the Software without restriction, including without limitation the rights to
// use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
// of the Software, and to permit persons to whom the Software is furnished to do
// so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all
// copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
// SOFTWARE.

import SwiftUI
import FeedbackServices

extension RateTheAppView {
    final class ViewModel: ObservableObject {
        @Published var text: String = ""
        @Published var smileyRating: SmileyRating?
        
        @Published var isSending: Bool = false
        @Published var didSend: Bool = false
        @Published var failedToSend: Bool = false

        @Published var validationFailed: Bool = false

        @DI private var feedbackService: FeedbackService
        private var cancelBag = CancelBag()

        let title = Localizable.more_rating_title()

        var canSend: Bool {
            text.isEmpty == false
        }

        init() {
            listenForTextChanges()
        }

        private func listenForTextChanges() {
            // Reset validationFailed on text field changes...
            $text
                .removeDuplicates()
                .sink(receiveValue: { [weak self] _ in self?.validationFailed = false })
                .store(in: &cancelBag)
        }

        func reset() {
            text = ""
            smileyRating = nil
            isSending = false
            didSend = false
            failedToSend = false
            validationFailed = false
        }

        func sendFeedback() {
            guard canSend else {
                validationFailed = true
                return
            }
            failedToSend = false
            didSend = false
            isSending = true
            feedbackService
                .send(rating: smileyRating?.rawValue, comment: text)
                .receiveOnMain()
                .sink(
                    receiveCompletion: { [weak self] completion in
                        self?.isSending = false
                        switch completion {
                        case .finished:
                            self?.didSend = true
                        case .failure:
                            self?.failedToSend = true
                        }
                    },
                    receiveValue: {  }
                )
                .store(in: &cancelBag)
        }
    }
}
