// The MIT License
//
// Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Copyright (C) 2020 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Permission is hereby granted, free of charge, to any person obtaining a copy of
// this software and associated documentation files (the "Software"), to deal in
// the Software without restriction, including without limitation the rights to
// use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
// of the Software, and to permit persons to whom the Software is furnished to do
// so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all
// copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
// SOFTWARE.

import SwiftUI
import Combine
import CMSData
import ErrorData

struct SettingsView: View, ScreenTrackable {
    @ObservedObject private var viewModel: ViewModel = ViewModel()

    private let rowHeight: CGFloat = Metrics.Sizes.navigationRowHeight

    var body: some View {
        ScrollView {
            CompatibleVStack {
                CircleImageHeaderView(
                    title: viewModel.title,
                    text: Localizable.more_settings_description(),
                    imageUrl: nil,
                    image: Image(R.image.icons.more.settings.name)
                )

                // MARK: - Privacy
                Section(
                    header: sectionHeader(title: Localizable.settings_privacy_title()),
                    content: {
                        SettingsSectionContentCard {
                            navigationRowForPage(
                                title: Localizable.settings_master_data_title(),
                                description: Localizable.settings_master_data_description(),
                                publisher: viewModel.pagesServices.fetchMasterData()
                            )
                            navigationRowForPage(
                                title: Localizable.settings_my_log_title(),
                                description: Localizable.settings_my_log_description(),
                                publisher: viewModel.pagesServices.fetchMyLog()
                            )
                            navigationRowForPage(
                                title: Localizable.settings_my_locking_title(),
                                description: Localizable.settings_my_locking_description(),
                                publisher: viewModel.pagesServices.fetchMyLocking()
                            )
                            navigationRowForPage(
                                title: Localizable.settings_privacy_policy_title(),
                                description: Localizable.settings_privacy_policy_description(),
                                publisher: viewModel.pagesServices.fetchPrivacyPolicy(),
                                insertDivider: false
                            )
                        }
                    }
                )

                // MARK: - Security
                if viewModel.isBiometricAvailable {
                    Section(
                        header: sectionHeader(title: Localizable.settings_security_title()),
                        content: {
                            SettingsSectionContentCard {
                                toggleRow(
                                    title: Localizable.settings_use_biometrics_title(viewModel.biometricName),
                                    description: nil,
                                    binding: Binding(get: { viewModel.isBiometricToggleSwitchOn }, set: { on in
                                        viewModel.isBiometricToggleSwitchOn = on
                                        if on {
                                            viewModel.showAlertToInitiateBiometricEnabling = true
                                        } else if viewModel.isBiometricEnabled == true {
                                            viewModel.disableBiometricLogin()
                                        }
                                    }),
                                    insertDivider: false
                                )
                            }
                        }
                    )
                }

                // MARK: - About the app
                Section(
                    header: sectionHeader(title: Localizable.settings_about_the_app_title()),
                    content: {
                        SettingsSectionContentCard {
                            navigationRowForPage(
                                title: Localizable.settings_read_about_the_app_title(),
                                description: nil,
                                publisher: viewModel.pagesServices.fetchAboutTheApp()
                            )
                            navigationRowForPage(
                                title: Localizable.settings_who_is_behind_the_app_title(),
                                description: nil,
                                publisher: viewModel.pagesServices.fetchBehindTheApp()
                            )
                            navigationRow(
                                title: Localizable.settings_consent_title(),
                                description: nil,
                                destination: SettingsConsentPageView(
                                    viewModel: SettingsPageView.ViewModel(
                                        title: Localizable.settings_consent_title(),
                                        pageFetcher: viewModel.pagesServices.fetchConsent()
                                    )
                                ),
                                insertDivider: false
                            )
                        }
                    }
                )

                PrimaryButton(
                    title: Localizable.settings_logout(),
                    action: viewModel.userContext.logout
                )
                .accessibility(identifier: "log_out")
                .padding(.vertical)

                Text(Localizable.settings_version(viewModel.appVersion, viewModel.appBuildNumber))
                    .applyTextStyle(fontStyle: .info, textColor: R.color.greyText.color())
            }
            .padding()
            .alertOnBackground(isPresented: $viewModel.showAlertToInitiateBiometricEnabling, alert: {
                biometricAlert()
            })
        }
        .sheetOnBackground(isPresented: $viewModel.shouldInitiateBiometricEnabling, content: {
            SettingsEnableBiometricLoginView()
        })
        .navigationBarTitle(Text(viewModel.title), displayMode: .inline)
        .screenTracking(screenTrackingName)
        .onAppear(perform: {
            viewModel.updateValues()
        })
    }

    @ViewBuilder
    private func navigationRowForPage(title: String, description: String?, publisher: AnyPublisher<Page, ServiceError>, insertDivider: Bool = true) -> some View {
        navigationRow(
            title: title,
            description: description,
            destination: SettingsPageView(
                viewModel: SettingsPageView.ViewModel(
                    title: title,
                    pageFetcher: publisher
                )
            ),
            insertDivider: insertDivider
        )
    }

    @ViewBuilder
    private func navigationRow<T: View>(title: String, description: String?, destination: T, insertDivider: Bool = true) -> some View {
        NavigationLink(
            destination: destination,
            label: {
                ListItemNavigationRowView(
                    title: title,
                    description: description
                )
                .frame(minHeight: rowHeight)
            })
        if insertDivider {
            DefaultDivider()
        }
    }

    @ViewBuilder
    private func sectionHeader(title: String) -> some View {
        HStack {
            Text(title)
                .applyTextStyle(fontStyle: .h2)
            Spacer(minLength: 0)
        }
    }

    @ViewBuilder
    private func toggleRow(title: String, description: String?, binding: Binding<Bool>, insertDivider: Bool = true) -> some View {
        ListItemToggleRowView(
            title: title,
            description: description,
            isOn: binding
        )
        .frame(minHeight: rowHeight)
        if insertDivider {
            DefaultDivider()
        }
    }

    private func biometricAlert() -> Alert {
        Alert(
            title: Text(viewModel.biometricName),
            message: Text(Localizable.ios_alert_biometric_on_message(viewModel.biometricName)),
            primaryButton: .default(
                Text(Localizable.ios_alert_biometric_on_confirm()),
                action: {
                    viewModel.shouldInitiateBiometricEnabling = true
                }),
            secondaryButton: .cancel(
                Text(Localizable.common_cancel()),
                action: {
                    withAnimation(.default) {
                        viewModel.isBiometricToggleSwitchOn = viewModel.isBiometricEnabled // reset
                    }
                })
        )
    }
}

struct SettingsView_Previews: PreviewProvider {
    static var previews: some View {
        SettingsView()
    }
}
