// The MIT License
//
// Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Copyright (C) 2020 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Permission is hereby granted, free of charge, to any person obtaining a copy of
// this software and associated documentation files (the "Software"), to deal in
// the Software without restriction, including without limitation the rights to
// use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
// of the Software, and to permit persons to whom the Software is furnished to do
// so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all
// copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
// SOFTWARE.

import SwiftUI
import TIM

extension SettingsEnableBiometricLoginView {
    final class ViewModel: ObservableObject, PinCodeLoginViewModel {

        @DI private var locationAuthentication: LocalAuthenticationContext
        @DI var timStorage: TIMDataStorage
        @DI var logger: Logger
        @DI var userContext: UserContextProtocol

        var biometricName: String {
            locationAuthentication.biometricName
        }

        @Published var pinCode: String = ""
        var pinCodePublisher: Published<String>.Publisher { $pinCode }

        @Published var isLoading: Bool = false
        @Published var shouldResetWrongPinCodeState: Bool = false
        var shouldResetWrongPinCodeStatePublisher: Published<Bool>.Publisher { $shouldResetWrongPinCodeState }
        @Published var wrongCode: Bool = false

        @Published var loginInvalidated: Bool = false
        @Published var genericError: Bool = false
        @Published var sessionExpired: Bool = false
        @Published var criticalUnexpectedError: Bool = false

        @Published var showSuccessAlert: Bool = false

        var cancelBag = CancelBag()

        init() {
            observeChanges()
        }

        func handleCompletedPIN(_ pin: String) {
            guard let userId = userContext.userId else {
                genericError = true
                return
            }
            isLoading = true
            timStorage.enableBiometricAccessForRefreshToken(password: pin, userId: userId)
                .receiveOnMain()
                .sink(
                    receiveCompletion: { [weak self] completion in
                        self?.isLoading = false
                        switch completion {
                        case .failure(let error):
                            self?.handleTimError(error)
                        case .finished:
                            self?.showSuccessAlert = true
                        }
                    },
                    receiveValue: {  _ in }
                )
                .store(in: &cancelBag)
        }
    }
}
