// The MIT License
//
// Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Copyright (C) 2020 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Permission is hereby granted, free of charge, to any person obtaining a copy of
// this software and associated documentation files (the "Software"), to deal in
// the Software without restriction, including without limitation the rights to
// use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
// of the Software, and to permit persons to whom the Software is furnished to do
// so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all
// copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
// SOFTWARE.

import SwiftUI

struct OverviewDateOfBirthView: View, ScreenTrackable {
    @ObservedObject var viewModel: ViewModel
    @Environment(\.presentationMode) var presentationMode

    var body: some View {
        NavigationView {
            VStack {
                Text(Localizable.overview_pick_date_of_birth_description())
                    .applyTextStyle(fontStyle: .body)
                DatePicker(
                    "",
                    selection: $viewModel.date,
                    in: ...Date(),
                    displayedComponents: .date
                )
                .compatibleDatePickerStyle()
                Spacer()
                if viewModel.showResetButton {
                    LoadingButton(
                        title: Localizable.overview_reset_date_of_birth_button(),
                        isLoading: viewModel.isLoadingReset,
                        backgroundColor: R.color.pinWrong.color(),
                        action: {
                            viewModel.showResetAlert = true
                        }
                    )
                    .padding(.bottom)
                }
                LoadingButton(
                    title: Localizable.overview_pick_date_of_birth_button(),
                    isLoading: viewModel.isLoadingSave,
                    action: viewModel.saveSelectedDate
                )
            }
            .padding()
            .alert(for: $viewModel.error)
            .alertOnBackground(isPresented: $viewModel.showResetAlert, alert: {
                Alert(
                    title: Text(Localizable.overview_reset_date_of_birth_alert_title()),
                    message: Text(Localizable.overview_reset_date_of_birth_alert_description()),
                    primaryButton: .default(Text(Localizable.common_cta_reset()), action: viewModel.resetDate),
                    secondaryButton: .cancel(Text(Localizable.common_regret()))
                )
            })
            .navigationBarTitle(Text(Localizable.overview_pick_date_of_birth_title()), displayMode: .inline)
            .navigationBarItems(trailing: CloseModalButton(action: {
                presentationMode.wrappedValue.dismiss()
            }))
            .protectedAppearanceEvents(
                onAppear: viewModel.reset,
                onDisappear: viewModel.reset
            )
            .screenTracking(screenTrackingName)
            .onReceive(viewModel.$didSave, perform: { didSave in
                if didSave {
                    presentationMode.wrappedValue.dismiss()
                }
            })
        }
    }
}

private extension DatePicker {
    @ViewBuilder
    func compatibleDatePickerStyle() -> some View {
        if #available(iOS 14, *) {
            self
                .datePickerStyle(GraphicalDatePickerStyle())
                .accentColor(R.color.primary.color())
        } else {
            self.datePickerStyle(WheelDatePickerStyle())
        }
    }
}

struct OverviewDateOfBirthView_Previews: PreviewProvider {
    static var previews: some View {
        OverviewDateOfBirthView(
            viewModel: OverviewDateOfBirthView.ViewModel()
        )
    }
}
