// The MIT License
//
// Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Copyright (C) 2020 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Permission is hereby granted, free of charge, to any person obtaining a copy of
// this software and associated documentation files (the "Software"), to deal in
// the Software without restriction, including without limitation the rights to
// use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
// of the Software, and to permit persons to whom the Software is furnished to do
// so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all
// copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
// SOFTWARE.

import Foundation
import ErrorData

extension OverviewPregnancyPickerView {
    final class ViewModel : ObservableObject {
        @Published var selectedItemId: String?
        @Published var isLoading: Bool = false
        @Published var didSave: Bool = false
        @Published var error: Error?

        @Published private (set) var sections: [RadioListPickerSection] = []

        private var cancelBag = CancelBag()

        @DI private var maternityEnvironmentStore: MaternityDataEnvironmentStore

        init() {
            reset()
        }

        func reset() {
            sections = [
                RadioListPickerSection(
                    id: "active",
                    title: Localizable.overview_pick_pregnancy_active_section(),
                    items: maternityEnvironmentStore.maternities?.filter({ !$0.isEnded }) ?? []
                ),
                RadioListPickerSection(
                    id: "ended",
                    title: Localizable.overview_pick_pregnancy_ended_section(),
                    items: maternityEnvironmentStore.maternities?.filter({ $0.isEnded }) ?? []
                )
            ]
            selectedItemId = maternityEnvironmentStore.selectedMaternity?.id
            error = nil
            isLoading = false
            didSave = false
        }

        func save() {
            guard selectedItemId != maternityEnvironmentStore.selectedMaternity?.id else {
                // Just finish if the user selects the one that already is selected.
                didSave = true
                return
            }
            isLoading = true

            if let maternity = maternityEnvironmentStore.maternities?.first(where: { $0.id == selectedItemId }) {
                maternityEnvironmentStore.changeSelectedMaternity(maternity)
                    .receiveOnMain()
                    .sink(
                        receiveCompletion: { [weak self] completion in
                            switch completion {
                            case .failure(let error):
                                self?.error = error
                            case .finished:
                                self?.didSave = true
                            }
                            self?.isLoading = false
                        },
                        receiveValue: { }
                    )
                    .store(in: &cancelBag)

            } else {
                error = ServiceError.unexpectedError(nil)
                isLoading = false
                didSave = false
            }
        }
    }
}
