// The MIT License
//
// Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Copyright (C) 2020 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Permission is hereby granted, free of charge, to any person obtaining a copy of
// this software and associated documentation files (the "Software"), to deal in
// the Software without restriction, including without limitation the rights to
// use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
// of the Software, and to permit persons to whom the Software is furnished to do
// so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all
// copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
// SOFTWARE.

import SwiftUI
import GMAppData

struct OverviewView: View, ScreenTrackable {
    @ObservedObject var viewModel = ViewModel()
    @EnvironmentObject var maternityEnvironmentStore: MaternityDataEnvironmentStoreImpl
    @DI private var logger: Logger

    var body: some View {
        NavigationView {
            ScrollView {
                ZStack(alignment: Alignment(horizontal: .center, vertical: .top)) {
                    // Using screen size instead of `GeometryReader` because it ruins the publishing of viewModel changes on iOS 13...
                    BubblesBackgroundView(size: UIScreen.main.bounds.size) 
                    CompatibleVStack(spacing: 20) {
                        HStack {
                            Spacer()
                            Button(
                                action: {
                                    viewModel.presentMaternityPicker = true
                                },
                                label: {
                                    Image(R.image.icons.overview.births.name)
                                        .renderingMode(.template)
                                        .foregroundColor(R.color.text.color())
                                })
                                .frame(size: 40)
                                .background(
                                    Circle()
                                        .fill(R.color.cardBackground.color())
                                        .applyDefaultShadow()
                                )
                                .accessibility(label: Text(Localizable.overview_pick_pregnancy_title()))
                                .hidden((maternityEnvironmentStore.maternities ?? []).count <= 1)

                        }

                        headerView
                        // We should only show articles if active
                        if !viewModel.pregnancyState.isEnded {
                            DataLoadingView(
                                state: viewModel.state,
                                retryAction: viewModel.fetchData,
                                content: {
                                    CompatibleVStack(spacing: 20) {
                                        OverviewKnowledgeSectionView(
                                            title: Localizable.overview_relevant_right_now_title(),
                                            actionTitle: Localizable.overview_find_more_knowledge_button(),
                                            action: {
                                                viewModel.pushKnowledgeSection = true
                                            },
                                            categories: viewModel.currentRelevantArticleResults
                                        )
                                        if viewModel.nextRelevantArticleResults.hasSearchResults {
                                            OverviewKnowledgeSectionView(
                                                title: Localizable.overview_relevant_next_week_title(),
                                                actionTitle: nil,
                                                action: nil,
                                                categories: viewModel.nextRelevantArticleResults
                                            )
                                        }
                                        FeedbackInvitationView(didPressButton: $viewModel.presentSendFeedback)
                                    }
                                })
                        }
                    }
                    .padding()
                }
                NavigationLink(
                    destination: KnowledgeContentView(preferredNavigationBarDisplayMode: .inline),
                    isActive: $viewModel.pushKnowledgeSection,
                    label: {
                        EmptyView()
                    }
                )
            }
            .navigationBarTitle(
                Text(viewModel.navigationBarTitle),
                displayMode: .automatic
            )
            .screenTracking(screenTrackingName)
            .protectedAppearanceEvents(onAppear: viewModel.fetchData)
            .sheetOnBackground(
                isPresented: $viewModel.presentDidGiveBirthModal,
                content: {
                    OverviewDateOfBirthView(viewModel: viewModel.dateOfBirthViewModel)
                }
            )
            .sheetOnBackground(
                isPresented: $viewModel.presentSendFeedback,
                content: {
                    NavigationView {
                        RateTheAppView(isPresentedAsSheet: $viewModel.presentSendFeedback)
                            .navigationBarItems(trailing: CloseModalButton(action: {
                                viewModel.presentSendFeedback = false
                            }))
                    }
                }
            )
            .sheetOnBackground(
                isPresented: $viewModel.presentMaternityPicker,
                content: {
                    OverviewPregnancyPickerView(
                        viewModel: viewModel.pregnancyPickerViewModel
                    )
                }
            )
        }
        .navigationViewStyle(StackNavigationViewStyle())
    }

    @ViewBuilder
    private var headerView: some View {
        switch viewModel.pregnancyState {
        case .none:
            OverviewNoCourseHeaderView(
                numberOfFetuses: viewModel.numberOfFetuses
            )
        case .activeAndWaitingForBirth:
            OverviewWaitingHeaderView(
                monthsIntoPregnancy: maternityEnvironmentStore.selectedMaternity?.months,
                dueDate: maternityEnvironmentStore.selectedMaternity?.dueDate ?? Date(),
                month: Int(maternityEnvironmentStore.selectedMaternity?.months ?? 0),
                week: maternityEnvironmentStore.selectedMaternity?.week ?? 0,
                gestationAge: maternityEnvironmentStore.selectedMaternity?.weekProgress ?? Localizable.common_missing_data(),
                trimester: maternityEnvironmentStore.selectedMaternity?.trimester ?? 0,
                numberOfFetuses: viewModel.numberOfFetuses,
                didGiveBirthButtonPressed: $viewModel.presentDidGiveBirthModal
            )
        case .activeAndGaveBirth:
            if let date = maternityEnvironmentStore.dateOfBirthForSelectedMaternity {
                OverviewGaveBirthStillActiveHeaderView(
                    dateOfBirth: date,
                    numberOfFetuses: viewModel.numberOfFetuses,
                    presentDateOfBirthModal: $viewModel.presentDidGiveBirthModal
                )
            } else {
                EmptyView()
                    .onAppear(perform: {
                        logger.error("Overview is requesting header view for `activeAndGaveBirth`, but does not have a birth date. This should never happen!")
                    })
            }
        case .endedButHaveNoBirthDate, .endedAndHaveBirthDate:
            #warning("Ended date?!")
            OverviewEndedHeaderView(
                endedDate: Date(),
                dateOfBirth: maternityEnvironmentStore.dateOfBirthForSelectedMaternity,
                numberOfFetuses: viewModel.numberOfFetuses,
                presentDateOfBirthModal: $viewModel.presentDidGiveBirthModal
            )
        }
    }
}

struct OverviewView_Previews: PreviewProvider {
    static var previews: some View {
        OverviewView(viewModel: OverviewView.PreviewViewModel())
            .environmentObject(MaternityDataEnvironmentStoreImpl.shared)
    }
}
