// The MIT License
//
// Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Copyright (C) 2020 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Permission is hereby granted, free of charge, to any person obtaining a copy of
// this software and associated documentation files (the "Software"), to deal in
// the Software without restriction, including without limitation the rights to
// use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
// of the Software, and to permit persons to whom the Software is furnished to do
// so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all
// copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
// SOFTWARE.

import Foundation
import MaternityServices
import GMAppData
import CMSServices
import CMSData

extension OverviewView {
    class ViewModel: ObservableObject {
        private var cancelBag = CancelBag()
        @DI private var maternityEnvironmentStore: MaternityDataEnvironmentStore
        @DI private var articlesService: ArticlesService

        // Kept in this view model to avoid creating new instances on EnvironmentObject changes...
        private (set) var pregnancyPickerViewModel = OverviewPregnancyPickerView.ViewModel()

        @Published var state: DataLoadingState = .initial
        @Published var presentDidGiveBirthModal: Bool = false
        @Published var presentSendFeedback: Bool = false
        @Published var presentMaternityPicker: Bool = false
        @Published var pushKnowledgeSection: Bool = false

        @Published var currentRelevantArticleResults: [CMSData.Category] = []
        @Published var nextRelevantArticleResults: [CMSData.Category] = []

        var navigationBarTitle: String {
            maternityEnvironmentStore.selectedMaternity?.recordTarget.name.fullName ?? Localizable.overview_welcome_message()
        }

        //Has to be kept here, to avoid recreation of view model on environment changes, while sheet is present.
        let dateOfBirthViewModel = OverviewDateOfBirthView.ViewModel()

        var pregnancyState: OverviewPregnancyState {
            if maternityEnvironmentStore.selectedMaternity?.expectedDateOfDeliverySection?.activeDueDate == nil {
                return .none
            } else if maternityEnvironmentStore.selectedMaternity?.isEnded == true {
                if maternityEnvironmentStore.dateOfBirthForSelectedMaternity == nil {
                    return .endedButHaveNoBirthDate
                } else {
                    return .endedAndHaveBirthDate
                }
            } else if maternityEnvironmentStore.dateOfBirthForSelectedMaternity == nil {
                return .activeAndWaitingForBirth
            } else {
                return .activeAndGaveBirth
            }
        }

        var numberOfFetuses: Int {
            maternityEnvironmentStore.selectedMaternity?.overallAssessmentSection?.prenatalCareAssessment.numberOfFetuses ?? 0
        }

        func fetchData() {
            guard state.allowsDataFetch else {
                return
            }
            state = .loading
            if let week = maternityEnvironmentStore.selectedMaternity?.week {
                articlesService.fetchArticleCategories(for: week)
                    .zip(articlesService.fetchArticleCategories(for: week + 1))
                    .receiveOnMain()
                    .sink(
                        receiveCompletion: { [weak self] completion in
                            switch completion {
                            case .failure(let error):
                                self?.state = .error(error)
                            case .finished:
                                let hasSearchResults = (self?.currentRelevantArticleResults.hasSearchResults ?? false) || (self?.nextRelevantArticleResults.hasSearchResults ?? false)
                                self?.state = hasSearchResults
                                    ? .data
                                    : .empty(Localizable.overview_empty_state_title(), Localizable.overview_empty_state_text())
                            }
                        },
                        receiveValue: { [weak self] (currentCategories, nextCategories) in
                            self?.currentRelevantArticleResults = currentCategories
                            self?.nextRelevantArticleResults = nextCategories
                        }
                    )
                    .store(in: &cancelBag)
            }
        }
    }

    final class PreviewViewModel: ViewModel {
        override func fetchData() {
            currentRelevantArticleResults = Array(CMSData.Category.mockList().prefix(2))
            nextRelevantArticleResults = [ CMSData.Category.mockList().last! ]
            state = .data
        }
    }
}
