// The MIT License
//
// Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Copyright (C) 2020 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Permission is hereby granted, free of charge, to any person obtaining a copy of
// this software and associated documentation files (the "Software"), to deal in
// the Software without restriction, including without limitation the rights to
// use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
// of the Software, and to permit persons to whom the Software is furnished to do
// so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all
// copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
// SOFTWARE.

import SwiftUI

struct OverviewHeaderValuesView: View {
    let controls: [OverviewHeaderControl]

    private let minValueHeight: CGFloat = 32

    var body: some View {
        HStack(spacing: 10) {
            ForEach(controls.indices, id: \.self) { index in
                let control = controls[index]
                if control.prefixSpace {
                    Spacer(minLength: 0)
                }
                view(for: control)
            }
        }
    }

    @ViewBuilder
    private func view(for control: OverviewHeaderControl) -> some View {
        switch control.type {
        case .value(let value):
            OverviewHeaderDefaultValueCard(
                title: control.title,
                value: value,
                minWidth: control.minWidth
            )
        case .dualValue(let value1, let value2):
            OverviewHeaderDualValueCard(
                title: control.title,
                value1: value1,
                value2: value2,
                minWidth: control.minWidth
            )
        case .textAction(let actionTitle, let action):
            OverviewHeaderTextActionCard(
                title: control.title,
                actionTitle: actionTitle,
                action: action ?? {},
                minWidth: control.minWidth
            )
            .disabled(action == nil)
        case .imageAction(let image, let action):
            OverviewHeaderImageActionCard(
                title: control.title,
                image: image,
                action: action ?? {},
                minWidth: control.minWidth
            )
            .disabled(action == nil)
        case .date(let date, let action):
            OverviewHeaderDateCard(
                title: control.title,
                date: date,
                minWidth: control.minWidth,
                action: action ?? {}
            )
            .disabled(action == nil)
        }
    }
}

struct OverviewHeaderValuesView_Previews: PreviewProvider {
    static var previews: some View {
        VStack {
            OverviewHeaderValuesView(
                controls: [
                    OverviewHeaderControl(
                        title: "Test",
                        type: .value("12"),
                        minWidth: 70
                    ),
                    OverviewHeaderControl(
                        title: "Test 2",
                        type: .value(nil),
                        minWidth: 30
                    ),
                    OverviewHeaderControl(
                        title: "Test 3",
                        type: .textAction("Lala", { }),
                        minWidth: 70
                    ),
                    OverviewHeaderControl(
                        title: "Test 4",
                        type: .dualValue("12", "(12+3)"),
                        minWidth: 70
                    )
                ]
            )
            OverviewHeaderValuesView(
                controls: [
                    OverviewHeaderControl(
                        title: "Test",
                        type: .imageAction(R.image.icons.bulb, { }),
                        minWidth: 70
                    ),
                    OverviewHeaderControl(
                        title: nil,
                        type: .value(nil),
                        minWidth: 30
                    ),
                    OverviewHeaderControl(
                        title: nil,
                        type: .date(Date(), { }),
                        minWidth: 30
                    ),
                    OverviewHeaderControl(
                        title: nil,
                        type: .date(nil, nil),
                        minWidth: 30
                    )
                ]
            )
        }.padding()
    }
}
