// The MIT License
//
// Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Copyright (C) 2020 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Permission is hereby granted, free of charge, to any person obtaining a copy of
// this software and associated documentation files (the "Software"), to deal in
// the Software without restriction, including without limitation the rights to
// use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
// of the Software, and to permit persons to whom the Software is furnished to do
// so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all
// copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
// SOFTWARE.

import SwiftUI

struct PartialSheetButton {
    let title: String
    let color: Color
    /// `nil` actions will close the sheet without doing anything else.
    let action: (() -> Void)?

    static func `default`() -> PartialSheetButton {
        PartialSheetButton(
            title: Localizable.common_cta_close(),
            color: R.color.infoButton.color(),
            action: nil
        )
    }
}

struct PartialSheetView<PresentedView: View>: View {
    @Environment(\.viewController) private var viewControllerHolder: UIViewController?

    let buttons: [PartialSheetButton]
    let presentedView: () -> PresentedView

    // We need to use this because an bool is just flipped and has no progress.
    // Thus the view controller dismiss animation will be called right away.
    @State private var presentingAnimationProgress: CGFloat = 0.0
    @State private var presentedViewSize: CGSize = .zero
    private var isPresented: Bool { presentingAnimationProgress >= 1.0 }

    var body: some View {
        GeometryReader { reader in
            ZStack {
                VisualBlurEffectView(style: .systemChromeMaterialDark)
                    .edgesIgnoringSafeArea(.all)
                    .opacity(self.isPresented ? 0.9 : 0)
                    .onTapGesture(perform: closeSheet)

                VStack(spacing: 0) {
                    Spacer()

                    ScrollView {
                        self.presentedView()
                            .padding()
                            .onSizeChanged(ContentSizeReaderKey.self, perform: { (contentSize: CGSize) in
                                presentedViewSize = contentSize
                            })
                    }
                    .frame(maxHeight: presentedViewSize.height)
                    .background(R.color.cardBackground.color())
                    .cornerRadius(Metrics.Sizes.cardCornerRadius)

                    VStack(spacing: 16) {
                        ForEach(buttons, id: \.title) { button in
                            buttonView(for: button)
                        }
                    }
                    .frame(maxWidth: .infinity)
                    .padding(.vertical)
                }
                .padding(.horizontal)
                .background(Color.clear)
                .offset(x: 0, y: self.isPresented ? 0 : reader.frame(in: .global).height)
            }
        }
        .onAppear {
            withAnimation(.spring()) {
                self.presentingAnimationProgress = 1
            }
        }
    }

    @ViewBuilder
    private func buttonView(for button: PartialSheetButton) -> some View {
        Button(
            action: { buttonPressed(for: button) },
            label: {
                HStack {
                    Spacer()
                    Text(button.title)
                        .applyTextStyle(fontStyle: .h4, textColor: .white)
                        .padding()

                    Spacer()
                }
                .background(button.color)
                .cornerRadius(8)
            })
            .onAnimationCompleted(for: presentingAnimationProgress) {
                guard !self.isPresented else { return } // To avoid it dismisses when
                viewControllerHolder?.dismiss(animated: false, completion: nil)
            }
    }

    private func buttonPressed(for button: PartialSheetButton) {
        button.action?()
        closeSheet()
    }

    private func closeSheet() {
        withAnimation {
            self.presentingAnimationProgress = 0
        }
    }
}

private struct ContentSizeReaderKey: SizeReaderKey {
    static var defaultValue: CGSize = .zero
}

struct PartialSheetView_Previews: PreviewProvider {
    static var previews: some View {
        PartialSheetView(
            buttons: [
                PartialSheetButton(
                    title: "Slet",
                    color: R.color.errorRed.color(),
                    action: nil
                ),
                PartialSheetButton(
                    title: "Luk",
                    color: R.color.infoButton.color(),
                    action: nil
                )
            ]) {
            InfoSheetContent(title: "Text", text: String.loremIpsum(.huuuuge))
        }
    }
}
