// The MIT License
//
// Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Copyright (C) 2020 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Permission is hereby granted, free of charge, to any person obtaining a copy of
// this software and associated documentation files (the "Software"), to deal in
// the Software without restriction, including without limitation the rights to
// use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
// of the Software, and to permit persons to whom the Software is furnished to do
// so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all
// copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
// SOFTWARE.

import XCTest
import DependencyInjection
import Combine
import MaternityServices
import UserSettingsServices
import Logger
import SharedServices
import GMAppData
import ConfigurationServices
@testable import MinGraviditet

class MaternityDataEnvironmentStoreImplTests: XCTestCase {
    private var envStore = MaternityDataEnvironmentStoreImpl.shared
    private let userSettingsService = UserSettingsServiceMock()
    private let configurationService = ConfigurationServiceMock()

    override func setUp() {
        super.setUp()

        envStore.clear()

        MockFlags.failRisk = .disabled

        Resolver.shared.allowOverride = true
        Resolver.shared.registerAsSingleton(DataCardService.self, DataCardServiceMock())
        Resolver.shared.registerAsSingleton(UserSettingsService.self, userSettingsService)
        Resolver.shared.registerAsSingleton(ConfigurationService.self, configurationService)
        Resolver.shared.registerAsSingleton(Logger.self, OSLogger())
    }

    func testInitialRequestUpdate() {
        userSettingsService.remove(key: .selectedEpisodeOfCareId)
        requestUpdate()
        // We should have the most recent maternity now...
        XCTAssertNotNil(envStore.selectedMaternity)
        XCTAssertEqual(envStore.maternities?.mostRecentDataCard()?.id, envStore.selectedMaternity?.id)
    }

    func testRequestUpdate() {
        var cancelBag = CancelBag()
        let date = Date()
        userSettingsService.save("3", for: .selectedEpisodeOfCareId)
        let setDateExpect = XCTestExpectation(description: "Setting date before requesting update ...")
        configurationService.updateDateOfBirth(maternityId: "3", date: date)
            .sink(receiveCompletion: { _  in}, receiveValue: { _ in setDateExpect.fulfill() })
            .store(in: &cancelBag)
        wait(for: [setDateExpect], timeout: 2.0)

        requestUpdate()

        // We should have the selected maternity with id "3" including date for it now...
        XCTAssertNotNil(envStore.selectedMaternity)
        XCTAssertNotEqual(envStore.maternities?.mostRecentDataCard()?.id, envStore.selectedMaternity?.id)
        XCTAssertEqual("3", envStore.selectedMaternity?.id)
        XCTAssertEqual(date, envStore.dateOfBirthForSelectedMaternity)
    }

    func testChangeSelectedMaternity() {
        var cancelBag = CancelBag()
        userSettingsService.remove(key: .selectedEpisodeOfCareId)
        requestUpdate()
        let selected = envStore.selectedMaternity!
        XCTAssertEqual(userSettingsService.get(key: .selectedEpisodeOfCareId), selected.id)

        let expect = XCTestExpectation(description: "Change selected")
        let changeToMaternity = envStore.maternities!.first(where: { $0.id != selected.id })!
        envStore.changeSelectedMaternity(changeToMaternity)
            .sink(receiveCompletion: { _ in }, receiveValue: { _ in expect.fulfill() })
            .store(in: &cancelBag)
        XCTAssertNotEqual(selected.id, envStore.selectedMaternity!.id)
        XCTAssertEqual(changeToMaternity.id, envStore.selectedMaternity!.id)
        XCTAssertEqual(userSettingsService.get(key: .selectedEpisodeOfCareId), changeToMaternity.id)
    }

    func testChangeDateOfBirthForSelected() {
        var cancelBag = CancelBag()
        userSettingsService.remove(key: .selectedEpisodeOfCareId)
        requestUpdate()
        XCTAssertNotNil(envStore.selectedMaternity)
        XCTAssertNil(envStore.dateOfBirthForSelectedMaternity)

        let autoSelectedDataCard = envStore.selectedMaternity!

        // Set date for auto selected maternity...
        let expect1a = XCTestExpectation(description: "Change date")
        let date1 = Date().addingTimeInterval(-3600)
        envStore.changeDateOfBirthForSelectedMaternity(date1)
            .sink(receiveCompletion: { _ in }, receiveValue: { _ in expect1a.fulfill() })
            .store(in: &cancelBag)
        wait(for: [expect1a], timeout: 2.0)
        XCTAssertNotNil(envStore.dateOfBirthForSelectedMaternity)
        XCTAssertEqual(date1, envStore.dateOfBirthForSelectedMaternity)

        // Change maternity to another and check that there is no date any more...
        let expect2a = XCTestExpectation(description: "Change maternity")
        let anotherDataCard = envStore.maternities!.first(where: { $0.id != envStore.selectedMaternity!.id })!
        envStore.changeSelectedMaternity(anotherDataCard)
            .sink(receiveCompletion: { _ in }, receiveValue: { expect2a.fulfill() })
            .store(in: &cancelBag)
        wait(for: [expect2a], timeout: 2.0)
        XCTAssertNil(envStore.dateOfBirthForSelectedMaternity)

        // Set date for newly selected maternity
        let expect2b = XCTestExpectation(description: "Change date")
        let date2 = Date().addingTimeInterval(-3600 * 12)
        XCTAssertNotEqual(date1, date2) // Sanity check ...
        envStore.changeDateOfBirthForSelectedMaternity(date2)
            .sink(receiveCompletion: { _ in }, receiveValue: { _ in expect2b.fulfill() })
            .store(in: &cancelBag)
        wait(for: [expect2b], timeout: 2.0)
        XCTAssertNotNil(envStore.dateOfBirthForSelectedMaternity)
        XCTAssertEqual(date2, envStore.dateOfBirthForSelectedMaternity)

        // Change maternity back to the auto selected and see that the date changes back.
        let expect1b = XCTestExpectation(description: "Change maternity")
        envStore.changeSelectedMaternity(autoSelectedDataCard)
            .sink(receiveCompletion: { _ in }, receiveValue: { expect1b.fulfill() })
            .store(in: &cancelBag)
        wait(for: [expect1b], timeout: 2.0)
        XCTAssertNotNil(envStore.dateOfBirthForSelectedMaternity)
        XCTAssertEqual(date1, envStore.dateOfBirthForSelectedMaternity)

        // Test clear
        let expect1c = XCTestExpectation(description: "Remove date")
        envStore.changeDateOfBirthForSelectedMaternity(nil)
            .sink(receiveCompletion: { _ in }, receiveValue: { _ in expect1c.fulfill() })
            .store(in: &cancelBag)
        wait(for: [expect1c], timeout: 2.0)
        XCTAssertNil(envStore.dateOfBirthForSelectedMaternity)
        XCTAssertNotEqual(date1, envStore.dateOfBirthForSelectedMaternity)
    }

    func testClear() {
        var cancelBag = CancelBag()
        requestUpdate()

        let expectDate = XCTestExpectation(description: "Date")
        envStore.changeDateOfBirthForSelectedMaternity(Date())
            .sink(receiveCompletion: { _ in }, receiveValue: { _ in expectDate.fulfill() })
            .store(in: &cancelBag)
        wait(for: [expectDate], timeout: 2.0)
        XCTAssertNotNil(envStore.selectedMaternity)
        XCTAssertNotNil(envStore.maternities)
        XCTAssertNotNil(envStore.dateOfBirthForSelectedMaternity)

        envStore.clear()
        XCTAssertNil(envStore.selectedMaternity)
        XCTAssertNil(envStore.maternities)
        XCTAssertNil(envStore.dateOfBirthForSelectedMaternity)
    }

    private func requestUpdate() {
        var cancelBag = CancelBag()
        let expect = XCTestExpectation()
        envStore.requestUpdate()
            .sink(receiveCompletion: { _ in expect.fulfill() }, receiveValue: { _ in })
            .store(in: &cancelBag)
        wait(for: [expect], timeout: 2.0)
    }
}
