// The MIT License
//
// Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Copyright (C) 2020 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
//
// Permission is hereby granted, free of charge, to any person obtaining a copy of
// this software and associated documentation files (the "Software"), to deal in
// the Software without restriction, including without limitation the rights to
// use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
// of the Software, and to permit persons to whom the Software is furnished to do
// so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all
// copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
// SOFTWARE.
import Foundation
import Combine
import SharedServices
import GMAppData
import ErrorData

public class ObservationServiceMock: ObservationService {

    public init() { }

    public func fetchObservations() -> AnyPublisher<[Observation], ServiceError> {
        let observations: [Observation] = (0...10).map({ i in
            Observation(
                dateOfMeasurement: Date(),
                values: (0...5).map({ v in ObservationValues(value: "value \(v)", unit: nil, name: "Titel \(v)")}),
                measuredBy: "Test Testsen",
                enteredBy: "Test Testsen",
                completed: true,
                fetus: Bool.random() ? "Foster 1" : nil,
                activityReference: nil,
                type: ObservationType.allCases.count - 1 > i ? ObservationType.allCases[i] : nil,
                id: "\(i)",
                dirty: nil
            )
        })
        return MockPublisher.mockFailable(
            value: observations,
            error: ServiceError.networkError(.missingInternetConnection)
        )
    }

    public func fetchObservationsTableScan(maternityId: String) -> AnyPublisher<ScanningObservationTable, ServiceError> {
        let table = ScanningObservationTable(
            mother: ScanningObservationTableMother(rows: [
                ScanningObservationTableMotherRows(
                    activityId: "1",
                    date: Date().addingTimeInterval(-3600 * 24 * 30),
                    gestationAge: "12+4",
                    weeksIntoPregnancy: 12.56,
                    amnioticFluid: "Jaja",
                    flow: "🌊",
                    cervix: NumericValueWithUnit(value: 111, unit: "u"),
                    dvp: NumericValueWithUnit(value: 222, unit: "a"),
                    conclusion: "Konklusionen blev ingenting! 💣"
                ),
                ScanningObservationTableMotherRows(
                    activityId: "2",
                    date: Date(),
                    gestationAge: "16+4",
                    weeksIntoPregnancy: 16.56,
                    amnioticFluid: "Nejnej",
                    flow: "0️⃣",
                    cervix: NumericValueWithUnit(value: 333, unit: "u"),
                    dvp: NumericValueWithUnit(value: 444, unit: "a"),
                    conclusion: nil
                )
            ]),
            fetuses: [
                ScanningObservationTableFetuses(
                    name: "Foster 1",
                    id: "1",
                    rows: [
                        ScanningObservationTableRows(
                            activityId: "1",
                            date: Date().addingTimeInterval(-3600 * 24 * 30),
                            gestationAge: "12+4",
                            weeksIntoPregnancy: 12.56,
                            ulWeight: NumericValueWithUnit(value: 123, unit: nil),
                            weightDeviation: NumericValueWithUnit(value: 321, unit: nil),
                            fetusPresentation: "H"
                        ),
                        ScanningObservationTableRows(
                            activityId: "2",
                            date: Date(),
                            gestationAge: "16+3",
                            weeksIntoPregnancy: 16.4,
                            ulWeight: NumericValueWithUnit(value: 111, unit: nil),
                            weightDeviation: NumericValueWithUnit(value: 222, unit: nil),
                            fetusPresentation: "H"
                        )
                    ]
                )
            ]
        )
        return MockPublisher.mockFailable(
            value: table,
            error: ServiceError.networkError(.missingInternetConnection)
        )
    }

    public func fetchObservationsTableClinical(maternityId: String) -> AnyPublisher<ObservationTable, ServiceError> {
        let table = ObservationTable(
            mother: ObservationTableMother(rows: [
                ObservationTableMotherRows(
                    activityId: "1",
                    date: Date().addingTimeInterval(-3600 * 24 * 30),
                    gestationAge: "12+4",
                    weeksIntoPregnancy: 12.56,
                    weight: NumericValueWithUnit(value: 111, unit: "kg"),
                    bloodPressure: "Jap/Jip",
                    urine: "🟡",
                    edema: nil,
                    pubismeasurement: NumericValueWithUnit(value: 111, unit: "cm")
                ),
                ObservationTableMotherRows(
                    activityId: "2",
                    date: Date(),
                    gestationAge: "16+4",
                    weeksIntoPregnancy: 16.56,
                    weight: NumericValueWithUnit(value: 90, unit: "kg"),
                    bloodPressure: "Jip/Jap",
                    urine: "🟢",
                    edema: nil,
                    pubismeasurement: NumericValueWithUnit(value: 120, unit: "cm")
                )
            ]),
            fetuses: [
                ObservationTableFetuses(
                    name: "Foster 1",
                    id: "1",
                    rows: [
                        ObservationTableRows(
                            activityId: "1",
                            date: Date().addingTimeInterval(-3600 * 24 * 30),
                            gestationAge: "12+4",
                            weeksIntoPregnancy: 12.56,
                            fetusPresentation: "H",
                            fetusWeightEstimate: NumericValueWithUnit(value: 1234, unit: "g"),
                            fetusActivity: nil
                        ),
                        ObservationTableRows(
                            activityId: "2",
                            date: Date(),
                            gestationAge: "16+4",
                            weeksIntoPregnancy: 16.56,
                            fetusPresentation: "H",
                            fetusWeightEstimate: NumericValueWithUnit(value: 2000, unit: "g"),
                            fetusActivity: nil
                        )
                    ]
                )
            ]
        )
        return MockPublisher.mockFailable(
            value: table,
            error: ServiceError.networkError(.missingInternetConnection)
        )

    }
}
